/*
 * Reader.h
 *
 *  Created on: 2020-9-7
 *      Author: derkiot
 */

#ifndef READER_H_
#define READER_H_
#include "stdint.h"

#ifdef __cplusplus
extern "C" {
#endif

#define	 SAM_V_FRAME_START_CODE 	0xBB
#define  SAM_V_INIT_COM 			0x32   	  /* 解析服务器开始请求解析命令 */
#define  SAM_V_APDU_COM 			0x33   	  /* 解析服务器APDU命令 */
#define  SAM_V_FINISH_COM 			0x34   	  /* 解析服务器解析完成命令 */
#define  SAM_V_ERROR_COM			0x35   	  /* 解析服务器解析错误命令 */
#define  SAM_V_GET_AES_KEY_COM 		0x36 /* 获取明文解谜密钥 */

/*******************************************************************************
*                                 通讯命令定义                                 *
*******************************************************************************/
#define GET_UID                     (0x01)   /*获取UID*/
#define GET_PICC_TYPE               (0x02)   /*获取卡片类型*/
#define M1_SAVE_KEY_A               (0x03)   /*向模块写入需要验证的密钥(A密钥)*/
#define M1_READ_BLOCK               (0x04)   /*M1卡读块*/
#define M1_WRITE_BLOCK              (0x05)   /*M1卡写块*/
#define M1_VALUE_INIT               (0x06)   /*M1卡增减值初始化*/
#define M1_VALUE_ADD                (0x07)   /*M1卡增值*/
#define M1_VALUE_SUB                (0x08)   /*M1卡减值*/
#define UL_READ_BLOCK               (0x09)   /*UL卡读块*/
#define UL_WRITE_BLOCK              (0x0A)   /*UL卡写块*/
#define M1_SAVE_KEY_B               (0x0B)   /*向模块写入需要验证的密钥(B密钥)*/
#define M1_SET_KEY_TYPE             (0x0C)   /*设置模块使用密钥的类型*/
#define B_READ_BLOCK                (0x0D)   /*ISO14443-B读块*/
#define B_WRITE_BLOCK               (0x0E)   /*ISO14443-B写块*/
#define M1_READ_ALL                 (0x0F)   /*M1卡读所有扇区/块*/
#define UL_READ_ALL                 (0x10)   /*UL卡读所有扇区/块*/
#define B_READ_ALL                  (0x11)   /*ISO14443-B卡读所有块*/
#define EEROM_READ                  (0x12)   /*读EEROM指令*/
#define EEROM_WRITE                 (0x13)   /*写EEROM指令*/
#define B_ACTIVIT                   (0x14)   /*ISO14443-B 卡激活指令*/
#define A_ACTIVIT                   (0x15)   /*ISO14443-A 卡片激活指令*/
#define B_PDU_COM                   (0x16)   /*ISO14443-B pdu指令接口*/
#define A_PDU_COM                   (0x17)   /*ISO14443-A pdu指令接口*/
#define PDU_DEACTIVIT               (0x18)   /*卡片断电指令*/
#define SAM_ACTIVIT                 (0x19)   /*SAM卡激活指令*/
#define SAM_PDU_COM                 (0x1a)   /*SAM卡PDU指令接口*/
#define SAM_DEACTIVIT               (0x1b)   /*SAM卡去激活指令*/
#define UL_FAST_READ                (0x1c)   /*UL卡FAST_READ指令*/
#define UL_FAST_WRITE               (0x1d)   /*UL卡快速写指令*/
#define GET_FINANCIAL_NUM           (0x1e)   /*获取银行卡卡号*/
#define B_GET_DN_NUM                (0x1f)   /*获取DN码*/
#define SRI512_READ_COM             (0x20)   /*SRI512读块*/
#define SRI512_WRITE_COM            (0x21)   /*SRI512写块*/
#define NFC_P2P_EXCHANGE            (0x22)   /*P2P数据交互通道*/
#define GET_ATR_CMD                 (0x23)   /*获取ATR*/
#define ENTER_UPDATE_CMD            (0x24)   /*进入固件升级模式*/
#define UL_EXCHANGE_CMD             (0x25)   /*UL卡指令交互*/ 

#define AL_ISO15693_READ_SINGLE_BLOCK      (0x90)   /*ISO15693读单个块数据*/
#define AL_ISO15693_READ_MULTIPLE_BLOCK    (0x91)   /*ISO15693读单个块数据*/
#define AL_ISO15693_WRITE_SINGLE_BLOCK     (0x92)   /*ISO15693写单个块数据*/
#define AL_ISO15693_WRITE_MULTIPLE_BLOCK   (0x93)   /*ISO15693写单个块数据*/
#define AL_ISO15693_LOCK_BLOCK             (0x94)   /*ISO15693锁块*/
#define AL_AUTO_SEARCH_CARD_SW             (0x95)   /*自动寻卡开关*/
#define AL_ISO15693_READ_ALL               (0x96)   /*ISO15693读所有块*/


#define CONFIG_UART_BAUD            (0xA0)   /*配置串口波特率指令*/
#define CONFIG_SYS_P                (0xA1)   /*配置系统参数*/
#define GET_CONFIG_SYS_P            (0xA2)   /*读取当前系统参数*/
#define SAVE_ACCOUNT_COM            (0xA3)   /*保存密钥信息*/
#define GET_VERSION                 (0xB0)   /*获取软件版本号*/
#define GET_HW_VERSION              (0xB1)   /*获取硬件版本号*/
#define OPEN_BEEP_CMD               (0xB2)   /*开启蜂鸣器指令*/
#define GET_BEEP_CMD                (0xB3)   /*获取蜂鸣器*/
#define UART_RESEND                 (0xB4)   /*重发上一次的数据*/

/*                                 卡片类型定义            
*******************************************************************************/
#define PICC_TYPE_M1                  (1)      /*M1卡*/
#define PICC_TYPE_UL                  (2)      /*UL卡*/
#define PICC_TYPE_B                   (3)      /*ISO14443-B卡*/
#define PICC_TYPE_ACPU                (4)      /*ISO14443-A CPU卡*/
#define PICC_TYPE_15693               (5)      /*ISO15693*/
#define PICC_TYPE_FELICA              (6)      /*FeliCa*/
#define PICC_TYPE_SRI512              (7)      /*SRI512*/
#define PICC_TYPE_COPY                (8)      /*复制卡*/
#define PICC_TYPE_DF                  (9)      /*DESFire卡*/
#define PICC_TYPE_IDENTIFY_CARD       (21)     /*身份证*/


#define DK_ANALYSIS_EXTRA_LEN		   3/*解析成功返回德科额外信息*/
#define DN_ANALYSIS_EXTRA_LEN		   10 /*身份证信息帧头、应答等信号*/

typedef enum{
	ANALYSIS_CONTINUTE = 1,
	ANALYSIS_OK = 0,
	ANALYSIS_FAIL = -1,
	ANALYSIS_TIMEOUT = -2,
	ANALYSIS_UART_ERROR = -3,
	ANALYSIS_NET_ERROR = -4,
}analysis_status;


typedef void (*LCDFUN) (uint8_t rate);
void set_debug_flag(uint8_t flag);
int card_restart_power_on(void);
int set_auto_search_card_para(uint8_t on_off_search, uint8_t sd, uint8_t sp, uint16_t timeout);
int get_auto_search_card_para(uint8_t *para) ;
int close_beep(void);
int oepn_beep(uint8_t delay, uint8_t time);
int get_devices_version(uint8_t *version) ;
int close_uart_device(void);
void card_type_printf(uint8_t card_type);
int get_identify_card_data_with_block(LCDFUN lcd_dis , uint8_t *dn_data, uint16_t dn_data_len, uint8_t *out_data, uint16_t *out_len) ;
int get_idcard_card_uid_with_block( uint8_t *card_type, uint8_t *uid, uint8_t *uid_len, uint16_t time_out);

int saveWlt2Bmp( char* wltBuffer,const char* bmpPath);
int saveWlt2BmpUseFork( char* wltBuffer,const char* bmpPath);
int continue_to_parse(uint8_t *initdata, uint16_t initdata_len, uint8_t *out_data, uint16_t *out_len) ;
void remote_get_img(void);

#if (LOCAL_DECODE_IMG == 0)
	void get_send_svr_first_pkg_data(uint8_t *out_data, uint16_t *out_data_len);
	void get_rcv_svr_end_pkg_data(uint8_t *out_data, uint16_t *out_data_len);
#endif

extern void *svr_ip_manager_thread(void *ptr);

#ifdef __cplusplus
}
#endif

#endif /* READER_H_ */
