#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include "Reader.h"
#include "stdint.h"
#include <stddef.h>

#include "uart.h"
#include "com_byte_manager.h"
#include "al_cpu_apdu.h"
#include "dk_utils.h"
#include "sdk_config.h"

//激活CPU A卡
int ativity_cpu_a(void) {
	int 		ret;
	uint8_t 	out_data[64] = {0};
	uint16_t	out_data_len = 0;
	uint8_t 	data[64];
	uint16_t	rcv_data_len = 0;
	
	out_data_len = data_framing_full(A_ACTIVIT, NULL, 0, out_data);
	dk_log_hex("out_data:", out_data, out_data_len);
	ret = uart_send_data(out_data, out_data_len);

	if(wait_uart_data(3) == 0) {
		ret = uart_read_data(data, 3, &rcv_data_len);

		if(ret == 0 && rcv_data_len == 3){
			if(data[0] == 0xAA && data[1] == 0x01 && data[2] == 0xFE) {
				return 0;
			}
		}
	}
	return -1;
}

/*******************************************************************************
*函数名：  al_apdu_com_exchange
*功能：apdu交换指令
*入口参数：apdu_com：A卡和B卡命令类型
*返回参数：0：成功，其他：失败
*作者：    Liang.xu
*******************************************************************************/
int al_apdu_com_exchange(uint8_t apdu_com, uint8_t *in_apdu_data, uint16_t in_apdu_data_len, uint8_t *out_apdu_data, uint16_t *out_apdu_data_len){
	int 		ret = 0;
	uint8_t 	rcv_uart_data[4096] = {0};
	uint16_t	rcv_uart_data_len = 0;
	uint8_t 	cmd_data[1024] = {0};
	uint16_t	cmd_len = 0;

	if(out_apdu_data == NULL || in_apdu_data_len > 512){
		return -1;
	}
	
	//组帧
	cmd_len = data_framing_full(apdu_com, in_apdu_data, in_apdu_data_len, cmd_data);
	dk_log_hex("apdu cmd", cmd_data, cmd_len);
	
	ret = uart_send_data(cmd_data, cmd_len); 
	if(ret < 0){
		return -1;
	}
	
	ret = wait_uart_data(3);
	if(ret == -1){
		return -1;
	}
	
	ret = uart_comm_rcv_listener(rcv_uart_data, &rcv_uart_data_len);
	
	if(ret == 0 && rcv_uart_data[0]== 0xAA && rcv_uart_data[2]== apdu_com){
		DEBUG_Printf("apdu success\r\n");
		*out_apdu_data_len = rcv_uart_data_len - 3;
		memcpy(out_apdu_data, rcv_uart_data+3, *out_apdu_data_len);
		return 0;
	}

	if(ret == 0 && rcv_uart_data[0]== 0xBB && rcv_uart_data[3]== apdu_com){
		DEBUG_Printf("BB back apdu success\r\n");
		*out_apdu_data_len = rcv_uart_data_len - 4;
		memcpy(out_apdu_data, rcv_uart_data+4, *out_apdu_data_len);
		return 0;
	}
	return -1;
}

//A卡随机数获取指令
void al_A_apdu_test(void){
	int ret = 0;
	
	uint8_t data[256] ={0};
	uint16_t data_len = 0;
	uint8_t get_random[] = {0x00, 0x84, 0x00, 0x00, 0x08};

	//关闭自动寻卡
	set_auto_search_card_para(0x00, 0x14, 0x76, 3);

	ativity_cpu_a();
	
	//数据交换
	ret = al_apdu_com_exchange(A_PDU_COM, get_random, sizeof(get_random), data, &data_len);
	if(ret == 0){
		printf("return apdu\r\n");
		for(int i = 0; i < data_len; i++){
			printf("%02x", data[i]);
		}
		printf("\r\n");
	}

	//打开自动寻卡
	set_auto_search_card_para(0xFF, 0x14, 0x76, 3);
}






