#include<string.h>
#include <stdio.h>
#include <unistd.h>
#include "Reader.h"
#include "stdint.h"
#include <stddef.h>

#include "uart.h"
#include "com_byte_manager.h"
#include "al_mifare_one.h"
#include "dk_utils.h"
#include "sdk_config.h"

uint8_t key[6] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
uint8_t write_block_data[16] = {0xAC, 0x11, 0x22, 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 , 0x99 , 0xAA , 0xBB , 0xCC , 0xDD , 0xEE , 0xFF};
uint8_t read_block_data[16] = {0};

/*******************************************************************************
*函数名：  al_mifare_save_key
*功能：往模块写入M1卡秘钥
*入口参数：key：秘钥
*返回参数：0：成功，其他：失败
*作者：    Liang.xu
*******************************************************************************/
int al_mifare_save_key(uint8_t key_type_cmd, uint8_t *key){
	int 		ret = 0;
	uint8_t 	rcv_uart_data[256] = {0};
	uint16_t	rcv_uart_data_len = 0;
	uint8_t 	cmd_data[64] = {0};
	uint16_t	cmd_len = 0;

	if(key == NULL){
		return -1;
	}

	//组帧
	cmd_len = data_framing_full(key_type_cmd, key, 6, cmd_data);
	dk_log_hex("mifare auth cmd", cmd_data, cmd_len);
	
	ret = uart_send_data(cmd_data, cmd_len);
	if(ret < 0){
		return -1;
	}

	ret = wait_uart_data(3);
	if(ret == -1){
		return -1;
	}
	
	ret = uart_comm_rcv_listener(rcv_uart_data, &rcv_uart_data_len);

	if(ret == 0 && rcv_uart_data_len){
		if(rcv_uart_data[0]== 0xAA && rcv_uart_data[1]== 0x01 && rcv_uart_data[2]== 0xFE){
			return 0;
		}
	}
	return -1;
}

/*******************************************************************************
*函数名：  al_mifare_write_block
*功能：M1卡写块
*入口参数：data：16字节长度数据
*返回参数：0：成功，其他：失败
*作者：    Liang.xu
*******************************************************************************/
int al_mifare_write_block(uint8_t block_addr, uint8_t *block_data){
	int 		ret = 0;
	uint8_t		in_data[17] = {0};
	uint8_t 	rcv_uart_data[256] = {0};
	uint16_t	rcv_uart_data_len = 0;
	uint8_t 	cmd_data[64] = {0};
	uint16_t	cmd_len = 0;

	if(block_data == NULL){
		return -1;
	}
	
	in_data[0] = block_addr;
	memcpy(in_data+1, block_data, 16);
	dk_log_hex("write cmd step 1:", in_data, sizeof(in_data));
	//组帧
	cmd_len = data_framing_full(M1_WRITE_BLOCK, in_data, sizeof(in_data), cmd_data);
	dk_log_hex("write addr cmd", cmd_data, cmd_len);
	
	ret = uart_send_data(cmd_data, cmd_len);
	if(ret < 0){
		return -1;
	}
	
	ret = wait_uart_data(3);
	if(ret == -1){
		return -1;
	}

	ret = uart_comm_rcv_listener(rcv_uart_data, &rcv_uart_data_len);

	if(ret == 0 && rcv_uart_data_len){
		if(rcv_uart_data[0]== 0xAA && rcv_uart_data[1]== 0x01 && rcv_uart_data[2]== 0xFE){
			return 0;
		}
	}
	return -1;
}

/*******************************************************************************
*函数名：  al_mifare_read_block
*功能：M1卡读块
*入口参数：data：16字节长度数据
*返回参数：0：成功，其他：失败
*作者：    Liang.xu
*******************************************************************************/
int al_mifare_read_block(uint8_t block_addr, uint8_t *block_data){
	int 		ret = 0;
	uint8_t	    in_data[1] = {0};
	uint8_t 	rcv_uart_data[256] = {0};
	uint16_t	rcv_uart_data_len = 0;
	uint8_t 	cmd_data[64] = {0};
	uint16_t	cmd_len = 0;

	if(block_data == NULL){
		return -1;
	}

	in_data[0] = block_addr;
	//组帧
	cmd_len = data_framing_full(M1_READ_BLOCK, in_data, sizeof(in_data), cmd_data);
	dk_log_hex("read addr cmd", cmd_data, cmd_len);
	
	ret = uart_send_data(cmd_data, cmd_len);
	if(ret < 0){
		return -1;
	}
	
	ret = wait_uart_data(3);
	if(ret == -1){
		return -1;
	}

	ret = uart_comm_rcv_listener(rcv_uart_data, &rcv_uart_data_len);

	if(ret == 0 && rcv_uart_data_len){
		if(rcv_uart_data[0]== 0xAA && rcv_uart_data[1]== 0x12 && rcv_uart_data[2]== M1_READ_BLOCK && rcv_uart_data[3] == block_addr){
			memcpy(block_data, rcv_uart_data+4, 16);
			return 0;
		}
	}
	return -1;
}

void al_mifare_one_test(void){
	int ret = 0;
	
	if(al_mifare_save_key(M1_SAVE_KEY_A, key) != 0){
		return;
	}
	//无论读写都需要设置秘钥
	ret = al_mifare_write_block(0x08, write_block_data);
	if(ret == 0){
		printf("write m1 data success\r\n");
	}
	ret = al_mifare_read_block(0x08, read_block_data);
	if(ret == 0){
		printf("read m1 data success\r\n");
		for(int i = 0; i < 16; i++){
			printf("%02x", read_block_data[i]);
		}
		printf("\r\n");
	}
}





