#include <stdio.h>
#include <stdlib.h>
#include "stdint.h"
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <errno.h>
#include <signal.h>
#include <pthread.h>
#include "uart.h"
#include "dk_utils.h"
#include "Reader.h"
#include "id_data_decode.h"
#include "al_cpu_apdu.h"
#include "al_mifare_one.h"
#include "sdk_config.h"
#include "YCTCard.h"

volatile static uint8_t  analysis_rate = 0;
char *serialName = "/dev/ttyS4";

//在显示屏上可以增加身份证解析进度,提醒用户不要马上拿开身份证
void schedule_printf(uint8_t rate)
{
	printf("\r\nanalysis rate : %%%d\r\n", rate );

}

/*******************************************************************************
*函数名：  identity_card_cloud_id_process
*功能：  身份证云解码处理过程  
*入口参数:无
*返回参数：0：成功，其他失败
*作者：    Lochy.Huang
*******************************************************************************/
int identity_card_cloud_id_process(uint8_t *dn_data, uint16_t dn_data_len){
	int 		ret = 0;
	uint8_t 	out_data[4096];
	uint16_t 	out_len;
	uint8_t  	id_info[256], id_bitmap[4096];
	uint16_t 	wordMsgBytesLen;
	uint16_t 	imageMsgBytesLen;
	uint16_t  	useful_message_index;
	uint8_t 	bcc_check = 0;
	
	system_time_printf();
	
	ret = get_identify_card_data_with_block(schedule_printf, dn_data, dn_data_len, out_data, &out_len);
	
	system_time_printf();
		
	switch (ret){
		case  ANALYSIS_OK:
			bcc_check = calc_lrc(out_data+5, out_len-5-1);
			printf("身份证解析成功\r\n");
			print_hex("out_data  ", out_data, out_len);

			if(out_data[0] == 0xAA && out_data[1] == 0xAA && out_data[2] == 0xAA && out_data[3] == 0x96 && 
				out_data[4] == 0x69 &&  bcc_check == out_data[out_len-1]){
				useful_message_index = DN_ANALYSIS_EXTRA_LEN;
				wordMsgBytesLen = ((out_data[useful_message_index] & 0xff) << 8) | (out_data[useful_message_index +1] & 0xff); 
				imageMsgBytesLen =   ((out_data[useful_message_index + 2] & 0xff) << 8) | (out_data[useful_message_index +3] & 0xff); 
				printf("wordMsgBytesLen:%d,imageMsgBytesLen:%d\r\n",wordMsgBytesLen,imageMsgBytesLen );	

					if(wordMsgBytesLen > 220) {
						memcpy(id_info, out_data + useful_message_index + 4, wordMsgBytesLen ); //拷贝文字信息
					}
					if(imageMsgBytesLen > 512) {
						memcpy(id_bitmap, out_data + useful_message_index + 4 + wordMsgBytesLen, imageMsgBytesLen ); //拷贝照片信息
					}
					system_time_printf();
					
					id_data_decode(id_info, wordMsgBytesLen, id_bitmap, imageMsgBytesLen);
						
					//到这一步才算解析完成，才能拿开身份证，此时打开蜂鸣器
					//open_beep();
					//beep_control(0x00C8, 0x0000, 2);
				}
				else {
					printf("dn data bcc check fail\r\n");
				}
				//card_restart_power_on();
				break;

			case ANALYSIS_NET_ERROR:
				printf("网络错误\r\n");
				//card_restart_power_on();
				break;
			case ANALYSIS_UART_ERROR:
				printf("UART错误\r\n");
				//card_restart_power_on();
				break;
			
			case  ANALYSIS_FAIL:
			default:
				printf("解析失败，请重新放卡\r\n");
//				card_restart_power_on();
				break;
		}
	return 0;
}

//读卡线程
void *read_card_thread(void *ptr) {


	int 		ret = 0;
	uint8_t		card_type  = 0;
	uint8_t		uid[255] = {0};
	uint8_t		uid_len = 0;
	uint8_t		version;
	uint8_t 	para[16] = {0};
	uint8_t		sd = 0; //非低功耗寻卡间隔
	uint8_t		sp = 0; //寻卡参数
	char 		YCT_json_str[512] = {0};

	//1:打开log,0:关闭log
	set_debug_flag(0);

	if(open_uart_device(serialName) == UART_INIT_FAIL) {
		printf("open %s fail\r\n", serialName);
		while (1);
	}

	if(get_devices_version(&version) == 0) {
		printf("softversion:%02x\r\n", version);
	}

	if(get_auto_search_card_para(para) == 0) {
		print_hex("para", para, 9);
		//SP参数,返回卡类型
		if((para[3] & (1<<4)) == 0){//未返回卡片类型
			sd = para[2];
			sp = para[3] | (1<<4);
			ret = set_auto_search_card_para(0xFF, sd, sp, 3); //超时时间为3秒
		}
	}
	//打开自动寻卡
	//set_auto_search_card_para(0xFF, 0x14, 0xF6, 3);

	
	#ifdef ME_ANT
	close_beep();
	#endif
	
	while(1){
		ret = get_idcard_card_uid_with_block(&card_type, uid, &uid_len, 2); //阻塞两秒
		if(ret != 0 || card_type == 0){
			printf("没有检测到卡片\r\n");
			//card_restart_power_on();
			continue;
		}
		if(ret == 0 && card_type != 0 && card_type != PICC_TYPE_IDENTIFY_CARD){
			card_type_printf(card_type);
			print_hex("卡号:", uid, uid_len);
		}
		printf("card_type:%d\r\n", card_type);
		switch (card_type){
			case PICC_TYPE_B: 
				break;

			case PICC_TYPE_M1: 
				//al_mifare_one_test();
				if(getYctMifareCardInfo(uid, YCT_json_str)){
					printf("YCT json数据为：%s",YCT_json_str );
				}
				break;

			case PICC_TYPE_ACPU: 
				al_A_apdu_test();
//				if(getYctCPUCardInfo(YCT_json_str)){
//					DEBUG_Printf("YCT json数据为：%s",YCT_json_str );
//				}
		
				//card_restart_power_on();
				break;
			case PICC_TYPE_IDENTIFY_CARD:
				identity_card_cloud_id_process(uid, uid_len);
				break;
			
			default:
				break;
		}
		
	}
	close_uart_device();
	return 0;

}

int main(int argc, char *argv[]) {

	int 		ret;
	pthread_t 	id1;
#if(DYNAMIC_IP)
	pthread_t	id2;
#endif

	ret = pthread_create(&id1, NULL, read_card_thread, NULL);
	if(ret){
		printf("Create read_card_thread error!\n");
		return 0;
	}
	
#if(DYNAMIC_IP)
	ret = pthread_create(&id2, NULL, svr_ip_manager_thread, NULL);
	if(ret){
		printf("Create svr_ip_manager_thread error!\n");
		return 0;
	}
	pthread_join(id2,NULL);
#endif

	pthread_join(id1,NULL);
	return 0;
}





