/*
 * Reader.h
 *
 *  Created on: 2020-9-7
 *      Author: derkiot
 */
#ifndef READER_H_
#define READER_H_
#include "stdint.h"
#include "dk_utils.h"
#ifdef __cplusplus
extern "C" {
#endif

#define	 USB_TIMEOUT	2000
#define	 SAM_V_FRAME_START_CODE 	0xBB
#define  SAM_V_INIT_COM 			0x32   	  	/* 解析服务器开始请求解析命令 */
#define  SAM_V_APDU_COM 			0x33   	  	/* 解析服务器APDU命令 */
#define  SAM_V_FINISH_COM 			0x34   	  	/* 解析服务器解析完成命令 */
#define  SAM_V_ERROR_COM			0x35   	  	/* 解析服务器解析错误命令 */
#define  SAM_V_GET_AES_KEY_COM 		0x36 	 	/* 获取明文解谜密钥 */

#define  GET_VERSION_COM			0x71		/* 获取版本号 */
#define  MIFARE_AUTH_COM			0x40		/* M1卡密钥验证 */
#define  MIFARE_APDU_COM		    0x41		/* M1卡写块和读块命令 */
#define  ACTIVATE_PICC_COM          0x62        /*激活卡片指令*/
#define  AUTO_SEARCH_CARD_COM       0x63        /*自动寻卡指令*/
#define  APDU_COM                   0x6F        /*apdu指令*/
#define  BPDU_COM                   0x7F        /* 身份证APDU指令通道*/
#define  AUTO_SEARCH_CARD_COM		0x63		/* 自动寻卡指令 */
#define  ANTENNA_OFF_COM            0x6E        /*关闭天线指令*/


#define DK_ANALYSIS_EXTRA_LEN		3			/*解析成功返回德科额外信息*/
#define DN_ANALYSIS_EXTRA_LEN		10 			/*身份证信息帧头、应答等信号*/

#define RF_NONE_CARD            	0       //RF通道中没有卡片
#define ISO14443_A              	1       //A卡
#define ISO14443_B              	2       //B卡
#define RF_TYPE_FELICA          	3
#define RF_TYPE_MF              	4
#define ISO15693                	5
#define RF_TYPE_UL              	6
#define RF_TYPE_DF              	7
#define SRI512                  	8
#define RF_TYPE_MF_MORE        	 	9      //检测到多张M1卡
#define RF_TYPE_T1T             	10
#define RF_TYPE_P2P             	11     //P2P

typedef enum{
	ANALYSIS_CONTINUTE = 1,
	ANALYSIS_OK = 0,
	ANALYSIS_FAIL = -1,
	ANALYSIS_TIMEOUT = -2,
	ANALYSIS_USB_ERROR = -3,
	ANALYSIS_NET_ERROR = -4,
}analysis_status;

typedef void (*LCDFUN) (uint8_t rate);
void set_debug_flag(uint8_t flag);
void offset_rate_value(void);
uint8_t cmd_response_check(uint8_t cmd, uint8_t *rsp_data );
void card_type_printf(uint8_t card_type);
bool is_sfz_card_confirm(uint8_t *b_card_type);
int card_restart_power_on(void);
int startAutoSearchCard(void);
int stopAutoSearchCard(void);
int get_auto_search_card_status(uint8_t *out_data, uint8_t *out_len, uint16_t timeout);
int get_idcard_card_uid_with_block( uint8_t *card_type, uint8_t *uid, uint8_t *uid_len, uint16_t time_out);
int get_version(uint8_t *version);
int continue_to_parse(uint8_t *initdata, uint16_t initdata_len, uint8_t *out_data, uint16_t *out_len);
int get_identify_card_data_with_block(LCDFUN lcd_dis , uint8_t *out_data, uint16_t *out_len);
int get_samv_id_data_with_block(uint8_t *out_data, uint16_t *out_len);
int beep_control(uint16_t open_time, uint16_t close_time, uint8_t cnt);
void get_send_svr_first_pkg_data(uint8_t *out_data, uint16_t *out_data_len);
void get_rcv_svr_end_pkg_data(uint8_t *out_data, uint16_t *out_data_len);
bool is_sfz_card_dn_confirm(void);

extern void *svr_ip_manager_thread(void *ptr);


#define DCTRI_RET_BASE          -0x6000
#define DCTRI_RET_OK            0
#define DCTRI_RET_ERR_PARA      -(DCTRI_RET_BASE + 1)
#define DCTRI_RET_ERR_RESP      -(DCTRI_RET_BASE + 2)
#define DCTRI_RET_ERR_COMM      -(DCTRI_RET_BASE + 3)
#define DCTRI_RET_ERR_DATA      -(DCTRI_RET_BASE + 4)
#define DCTRI_RET_ERR_OVERFLOW  -(DCTRI_RET_BASE + 5)
#define DCTRI_RET_ERR_NOCARD    -(DCTRI_RET_BASE + 6)
void remote_get_img(void);
#ifdef __cplusplus
}
#endif

#endif /* READER_H_ */
