#include<string.h>
#include <stdio.h>
#include <unistd.h>
#include "usb_hid.h"
#include "Reader.h"
#include "stdint.h"
#include <stddef.h>

#include "al_mifare_one.h"
#include "ComByteManager.h"
#include "dk_utils.h"
#include "sdk_config.h"

uint8_t key[6] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};
uint8_t write_block_data[16] = {0x00, 0x11, 0x22, 0x33 , 0x44 , 0x55 , 0x66 , 0x77 , 0x88 , 0x99 , 0xAA , 0xBB , 0xCC , 0xDD , 0xEE , 0xFF};
uint8_t read_block_data[16] = {0};


/*******************************************************************************
*函数名：  al_mifare_key_auth
*功能：M1卡秘钥验证
*入口参数：key：秘钥
*返回参数：0：成功，其他：失败
*作者：    Liang.xu
*******************************************************************************/
int al_mifare_key_auth(uint8_t block_addr, uint8_t key_type, uint8_t *key, uint8_t *uid){
	int 		ret = 0;
	uint8_t	    in_data[2+6+4] = {0};
	uint8_t 	data[256] = {0};
	uint16_t	len = 0;
	uint8_t 	cmd_data[64] = {0};
	uint16_t	cmd_len = 0;

	if(key == NULL || uid == NULL){
		return -1;
	}

	in_data[0] = block_addr;
	in_data[1] = key_type;
	memcpy(in_data+2, key, 6);
	memcpy(in_data+2+6, uid, 4);
	dk_log_hex("mifare auth cmd", in_data, sizeof(in_data));
	//组帧
	cmd_len = data_framing_full(MIFARE_AUTH_COM, in_data, sizeof(in_data), cmd_data);
	dk_log_hex("mifare auth cmd", cmd_data, cmd_len);
	
	ret = sendDataLen(cmd_data, cmd_len, USB_TIMEOUT);
	if(ret < 0){
		return -1;
	}
	
	ret = hid_get_data(data, &len, USB_TIMEOUT);
	if(ret == 0){
		dk_log_hex("auth return:", data, len);
	}
	
	if(ret == 0 && (cmd_response_check(MIFARE_AUTH_COM+1, data) == true)){
		DEBUG_Printf("auth success\r\n");
		return 0;
	}
	return -1;
}

/*******************************************************************************
*函数名：  al_mifare_write_block
*功能：M1卡写块
*入口参数：data：16字节长度数据
*返回参数：0：成功，其他：失败
*作者：    Liang.xu
*******************************************************************************/
int al_mifare_write_block(uint8_t block_addr, uint8_t *block_data){
	int 		ret = 0;
	uint8_t	    in_data[2] = {0xA0};
	uint8_t 	data[256] = {0};
	uint16_t	len = 0;
	uint8_t 	cmd_data[64] = {0};
	uint16_t	cmd_len = 0;

	if(data == NULL){
		return -1;
	}

	in_data[1] = block_addr;
	dk_log_hex("write cmd step 1:", in_data, sizeof(in_data));
	//组帧
	cmd_len = data_framing_full(MIFARE_APDU_COM, in_data, sizeof(in_data), cmd_data);
	dk_log_hex("write addr cmd", cmd_data, cmd_len);
	
	ret = sendDataLen(cmd_data, cmd_len, USB_TIMEOUT);
	if(ret < 0){
		return -1;
	}
	
	ret = hid_get_data(data, &len, USB_TIMEOUT);
	if(ret == 0){
		dk_log_hex("write addr return:", data, len);
	}
	
	if(ret != 0 || (cmd_response_check(MIFARE_APDU_COM+1, data) == false)){
		DEBUG_Printf("write addr fail\r\n");
		return -1;
	}

	//组帧
	cmd_len = data_framing_full(MIFARE_APDU_COM, block_data, 16, cmd_data);
	dk_log_hex("write data cmd", cmd_data, cmd_len);
	
	ret = sendDataLen(cmd_data, cmd_len, USB_TIMEOUT);
	if(ret < 0){
		return -1;
	}
	
	ret = hid_get_data(data, &len, USB_TIMEOUT);
	if(ret == 0){
		dk_log_hex("write data return:", data, len);
	}
	
	if(ret == 0 &&  (cmd_response_check(MIFARE_APDU_COM+1, data) == true)){
		DEBUG_Printf("write data success\r\n");
		return 0;
	}
	
	return -1;
}

/*******************************************************************************
*函数名：  al_mifare_read_block
*功能：M1卡读块
*入口参数：data：16字节长度数据
*返回参数：0：成功，其他：失败
*作者：    Liang.xu
*******************************************************************************/
int al_mifare_read_block(uint8_t block_addr, uint8_t *block_data){
	int 		ret = 0;
	uint8_t	    in_data[2] = {0x30};
	uint8_t 	data[256] = {0};
	uint16_t	len = 0;
	uint8_t 	cmd_data[64] = {0};
	uint16_t	cmd_len = 0;

	if(data == NULL){
		return -1;
	}

	in_data[1] = block_addr;
	//组帧
	cmd_len = data_framing_full(MIFARE_APDU_COM, in_data, sizeof(in_data), cmd_data);
	dk_log_hex("read addr cmd", cmd_data, cmd_len);
	
	ret = sendDataLen(cmd_data, cmd_len, USB_TIMEOUT);
	if(ret < 0){
		return -1;
	}
	
	ret = hid_get_data(data, &len, USB_TIMEOUT);
	if(ret == 0){
		dk_log_hex("read addr return:", data, len);
	}
	
	if(ret == 0 && (cmd_response_check(MIFARE_APDU_COM+1, data) == true)){
		memcpy(block_data, data+3, 16);
		DEBUG_Printf("read data success\r\n");
		return 0;
	}
	
	return -1;
}



void al_mifare_one_test(uint8_t *uid){
	int ret = 0;

	if(al_mifare_key_auth(0x08, 1, key, uid) != 0){
		return;
	}
	//无论读写都需要验证
	ret = al_mifare_write_block(0x08, write_block_data);
	if(ret == 0){
		printf("write m1 data success\r\n");
	}
	ret = al_mifare_read_block(0x08, read_block_data);
	if(ret == 0){
		printf("read m1 data success\r\n");
		for(int i = 0; i < 16; i++){
			printf("%02x", read_block_data[i]);
		}
		printf("\r\n");
	}
}




