/*
 * main.c
 * Created on: 2020.12.21
 * Author: derkiot
 */
/************************************************************
*更改为只通过一个域名访问，域名已做负载均衡
*************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "stdint.h"
#include <string.h>
#include <sys/time.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>
#include <errno.h>
#include <signal.h>
#include <pthread.h>
#include "usb_hid.h"
#include "ComByteManager.h"
#include "al_mifare_one.h"
#include "Reader.h"
#include "utf.h"
#include "dk_utils.h"
#include "id_data_decode.h"
#include "sdk_config.h"
#include "al_cpu_apdu.h"
#include "YCTCard.h"

//在显示屏上可以增加身份证解析进度,提醒用户不要马上拿开身份证
void schedule_printf(uint8_t rate)
{
	printf("analysis rate : %%%d\r\n", rate );
}

/*******************************************************************************
*函数名：  identity_card_cloud_id_process
*功能：  身份证云解码处理过程  
*入口参数:无
*返回参数：0：成功，其他失败
*作者：    Lochy.Huang
*******************************************************************************/
int identity_card_cloud_id_process(void){
	int 		ret = 0;
	uint8_t 	out_data[4096];
	uint16_t 	out_len;
	uint8_t  	id_info[256], id_bitmap[4096];
	uint16_t 	wordMsgBytesLen;
	uint16_t 	imageMsgBytesLen;
	uint16_t  	useful_message_index;
	uint8_t 	bcc_check = 0;
	
	system_time_printf();
	
	//阻塞等待，超时2秒
	ret = get_identify_card_data_with_block(schedule_printf, out_data, &out_len);
	
	system_time_printf();
		
	switch (ret){
		case  ANALYSIS_OK:
			bcc_check = calc_lrc(out_data+5, out_len-5-1);
			printf("身份证解析成功\r\n");
			dk_log_hex("out_data  ", out_data, out_len);

			if(out_data[0] == 0xAA && out_data[1] == 0xAA && out_data[2] == 0xAA && out_data[3] == 0x96 && 
				out_data[4] == 0x69 &&  bcc_check == out_data[out_len-1]){
				useful_message_index = DN_ANALYSIS_EXTRA_LEN;
				wordMsgBytesLen = ((out_data[useful_message_index] & 0xff) << 8) | (out_data[useful_message_index +1] & 0xff); 
				imageMsgBytesLen =   ((out_data[useful_message_index + 2] & 0xff) << 8) | (out_data[useful_message_index +3] & 0xff); 
				printf("wordMsgBytesLen:%d,imageMsgBytesLen:%d\r\n",wordMsgBytesLen,imageMsgBytesLen );	

					if(wordMsgBytesLen > 220) {
						memcpy(id_info, out_data + useful_message_index + 4, wordMsgBytesLen ); //拷贝文字信息
					}
					if(imageMsgBytesLen > 512) {
						memcpy(id_bitmap, out_data + useful_message_index + 4 + wordMsgBytesLen, imageMsgBytesLen ); //拷贝照片信息
					}
					system_time_printf();
					
					id_data_decode(id_info, wordMsgBytesLen, id_bitmap, imageMsgBytesLen);
						
					//到这一步才算解析完成，才能拿开身份证，此时打开蜂鸣器
					//open_beep();
					//beep_control(0x00C8, 0x0000, 2);
				}
				else {
					printf("dn data bcc check fail\r\n");
				}
				//card_restart_power_on();
				break;

			case ANALYSIS_NET_ERROR:
				printf("网络错误\r\n");
				//card_restart_power_on();
				break;
			case ANALYSIS_USB_ERROR:
				printf("USB错误\r\n");
				//card_restart_power_on();
				break;
	
			case  ANALYSIS_FAIL:
			default:
				printf("解析失败，请重新放卡\r\n");
				//card_restart_power_on();
				break;
		}
	return 0;
}

 /*  Created on: 2020-9-7
 *      Author: derkiot
 */
void *read_card_thread(void *ptr) {
	int 		ret;
	uint8_t 	version;
	uint8_t		cnt = 0;
	uint8_t		card_type  = 0;
	uint8_t		uid[32] = {0};
	uint8_t		uid_len = 0;
	uint8_t     b_card_type = 0;
	char 		YCT_json_str[512] = {0};

	set_debug_flag(0);
	
	while(1) {
		ret = hid_comm_init();  //usb hid init
		if(ret == 0 ) {
			printf("open usb hid device success\r\n");
			break;
		}
		else {
			printf("open usb hid device fail\r\n");
			Sleep(1000);
		}	
	}
	
	do{
		ret = get_version(&version);
		
		if(ret == true){
			printf("version:%02x\r\n", version);
		}
		else {
			printf("get version fail\r\n");
			Sleep(1000);
		}
	}while(ret != true && cnt++ < 5);

	while(1){
		ret = get_idcard_card_uid_with_block( &card_type, uid, &uid_len, 2000);
		if(ret != 0){
			if ( ret == LIBUSB_ERROR_OTHER || ret == LIBUSB_ERROR_NO_DEVICE || ret == LIBUSB_ERROR_NOT_FOUND || ret== LIBUSB_ERROR_UNNIT) { //usb device abnormal
				hid_comm_deinit(); //close USB HID
				printf("hid_comm_deinit\r\n");
				Sleep(1000);
				
				ret = hid_comm_init();	//usb hid init
				if(ret == 0 ) {
					printf("open usb hid device success\r\n");
				}
				else {
					printf("open usb hid device fail\r\n");
					Sleep(1000);
				}	
			}
			else if(ret == LIBUSB_ERROR_TIMEOUT){
				printf("没有检测到卡片\r\n");
			}
			continue;
		}

		if(ret == 0 && card_type != 0){
			card_type_printf(card_type);
			dk_log_hex("uid:", uid, uid_len);
		}

		switch (card_type){
			case ISO14443_B: 
				if(is_sfz_card_confirm(&b_card_type) == true && is_sfz_card_dn_confirm()){
					identity_card_cloud_id_process();
//					if(b_card_type == 0x01){
//						printf("normal B card\r\n");
//					}
//					else if(b_card_type == 0x02){
//						identity_card_cloud_id_process();
//					}
//					else {
//						printf("not normal B card\r\n");
//					}
				}
				break;

			case RF_TYPE_MF: 
				//al_mifare_one_test(uid);
				if(getYctMifareCardInfo(uid, YCT_json_str)){
					DEBUG_Printf("YCT json数据为：%s",YCT_json_str );
				}
				break;

			case ISO14443_A:
				if(getYctCPUCardInfo(YCT_json_str)){
					DEBUG_Printf("YCT json数据为：%s",YCT_json_str );
				}
				//al_A_apdu_test();
				break;
			
			default:
				break;
		}
	
	}
}
int main(int argc, char *argv[]) {

	int 		ret;
	pthread_t 	id1;
#if(DYNAMIC_IP)
	pthread_t	id2;
#endif

	ret = pthread_create(&id1, NULL, read_card_thread, NULL);
	if(ret){
		printf("Create read_card_thread error!\n");
		return 0;
	}
	
#if(DYNAMIC_IP)
	ret = pthread_create(&id2, NULL, svr_ip_manager_thread, NULL);
	if(ret){
		printf("Create svr_ip_manager_thread error!\n");
		return 0;
	}
	pthread_join(id2,NULL);
#endif

	pthread_join(id1,NULL);
	return 0;
}


